#!/usr/bin/env python3
"""The chat WOULD be watching."""

# SPDX-FileCopyrightText: 2025 NexusSfan <nexussfan@duck.com>
# SPDX-License-Identifier: GPL-3.0-or-later

import argparse
import os
from configparser import ConfigParser
import json
import pyhangouts2

if __name__ != "__main__":
    raise RuntimeError("This file is not meant to be imported.")


def parse_args():
    parser = argparse.ArgumentParser(description="The chat WOULD be watching.")
    parser.add_argument("--headless", action="store_true", help="Run in headless mode?")
    parser.add_argument(
        "-c", "--config", type=str, help="Config file to use.", default="config.ini"
    )
    parser.add_argument(
        "-l",
        "--log",
        type=str,
        help="Log file location",
        default="thechatwouldbewatching.log",
    )
    parser.add_argument(
        "-j",
        "--json",
        type=str,
        help="JSON file location",
        default="thechatwouldbewatching.json",
    )
    parser.add_argument(
        "-v",
        "--version",
        action="version",
        version=f"thechatwouldbewatching {pyhangouts2.__version__}",
    )
    args = parser.parse_args()
    return args


def log(message, logfile, jsonfile):
    with open(logfile, "a", encoding="utf-8") as f:
        f.write(
            f'[INFO] {message.name}: "{message.message.author}": {message.message.message}\n'
        )
    with open(jsonfile, "r", encoding="utf-8") as jr:
        jsonread = jr.read()
    with open(jsonfile, "w", encoding="utf-8") as j:
        js = json.loads(jsonread)
        js["log"].append(
            {
                "space": message.name,
                "message": message.message.message,
                "author": message.message.author,
            }
        )
        j.write(
            json.dumps(
                js,
                indent=4,
            )
        )


def inplus(message, no):
    for inp in no:
        if (
            f'{inp.name}: "{inp.message.author}": {inp.message.message}'
            == f'{message.name}: "{message.message.author}": {message.message.message}'
        ):
            return True


with open("thechatwouldbewatching.json", "w", encoding="utf-8") as j_init:
    j_init.write(
        json.dumps(
            {"log": []},
            indent=4,
        )
    )
with open("thechatwouldbewatching.log", "w", encoding="utf-8") as f_init:
    f_init.write("")

arguments = parse_args()

hangouts = pyhangouts2.PyHangouts2(headless=arguments.headless)

config = ConfigParser()

login_config = {}

# check if config exists
if os.path.exists(arguments.config):
    config.read(arguments.config)

    login_config = dict(dict(config).get("login", {}))

hangouts.login(
    login_config.get("method", "manual"),
    login_config.get("username"),
    login_config.get("password"),
)

notwatching = hangouts.ls()

if __name__ == "__main__":
    while True:
        watching = hangouts.ls()
        for i in watching:
            if not inplus(i, notwatching):
                log(i, arguments.log, arguments.json)
        notwatching = watching



"""PyHangouts2: A Selenium-based Google Chat library."""

# SPDX-FileCopyrightText: 2025 NexusSfan <nexussfan@duck.com>
# SPDX-License-Identifier: GPL-3.0-or-later

from warnings import warn
import time
from selenium import webdriver
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
import bs4

__version__ = "1.1.0"


class PyHangouts2:
    def __init__(self, headless=False):
        self.options = webdriver.ChromeOptions()
        if headless:
            warn("Headless mode may not work properly.", FutureWarning)
            self.options.add_argument("--headless=new")
        self.options.add_argument("--disable-blink-features=AutomationControlled")
        self.options.add_experimental_option("excludeSwitches", ["enable-automation"])
        self.options.add_experimental_option("useAutomationExtension", False)
        self.driver = webdriver.Chrome(options=self.options)
        self.driver.minimize_window()
        self.is_in_space = False
        self.is_logged_in = False
        self.space = None

    def login(self, method="manual", username=None, password=None):
        """Log in"""
        if self.is_logged_in:
            raise StatusError("User is already logged in")
        self.driver.maximize_window()
        if method == "auto":
            warn("Automatic login may not work properly.", FutureWarning)
            if not username or not password:
                raise AuthorizationError("Authentication info not completely provided.")
            self.driver.get("https://mail.google.com/chat/u/0/#chat/home")
            actions = ActionChains(self.driver)
            actions.send_keys(username).send_keys(Keys.ENTER).perform()
            time.sleep(3)
            actions.send_keys(password).send_keys(Keys.ENTER).perform()
            WebDriverWait(self.driver, 60).until(
                EC.url_matches("https://mail.google.com/chat/u/0/#chat/home")
            )
        elif method == "manual":
            self.driver.get("https://mail.google.com/chat/u/0/#chat/home")
            WebDriverWait(self.driver, 60).until(
                EC.url_matches("https://mail.google.com/chat/u/0/#chat/home")
            )
        else:
            raise SyntaxError("No valid login method provided!")
        self.driver.minimize_window()
        self.is_logged_in = True

    def join(self, space):
        """Join space"""
        if not self.is_logged_in:
            raise StatusError("User is not logged in")
        space_url = f"https://mail.google.com/chat/u/0/#chat/{space}"
        self.driver.get(space_url)
        WebDriverWait(self.driver, 60).until(EC.url_matches(space_url))
        self.driver.switch_to.frame(
            self.driver.find_element(by=By.NAME, value="single_full_screen")
        )
        time.sleep(1)
        self.is_in_space = True
        self.space = space

    def leave(self):
        """Leave space"""
        if not self.is_in_space:
            raise StatusError("User is not logged in")
        self.driver.switch_to.default_content()
        self.driver.get("https://mail.google.com/chat/u/0/#chat/home")
        WebDriverWait(self.driver, 60).until(
            EC.url_matches("https://mail.google.com/chat/u/0/#chat/home")
        )
        self.is_in_space = False
        self.space = None

    def ls(self):
        """List DMs and spaces"""
        if self.is_in_space:
            raise StatusError("User is in a space")
        self.driver.switch_to.frame(
            self.driver.find_element(by=By.NAME, value="single_full_screen")
        )
        page_html = self.driver.page_source
        soup = bs4.BeautifulSoup(page_html, "lxml")
        spaces = []
        spacesget = soup.find(
            "div",
            attrs={
                "jsname": "WbL0Ac",
                "role": "list",
                "aria-labelledby": "EWK8Bb-tJHJj",
                "jsaction": "UEEsmf:xO5IE;",
            },
        )
        for i in spacesget.contents:
            spaceid = i.attrs["id"].replace("/SCcFR", "")
            spaceattrs = (
                i.find("div")
                .find("div")
                .find(lambda tag: tag.name == "div" and "jsname" not in tag.attrs)
                .find("div", class_="Tcg1Uc")
                .find("div", class_="zeiL7e")
            )
            spacename = (
                spaceattrs.find("div", class_="WcXjib")
                .find("div", class_="Vb5pDe")
                .string
            )
            spacemsgs = spaceattrs.find("div", class_="ERFjwe").find(
                lambda tag: tag.name == "span" and "Hkj4n" in tag.attrs["class"]
            )
            spacemsg = ""
            spacemsgauthor = ""
            for msg in spacemsgs.contents:
                if isinstance(msg, bs4.element.Tag):
                    spacemsgauthor = msg.string
                    if spacemsgauthor:
                        spacemsgauthor = spacemsgauthor.strip()[:-1]
                if isinstance(msg, bs4.element.NavigableString):
                    spacemsg = msg
            if not spacemsgauthor:
                # in a dm, if other person sends message it doesn't show author
                spacemsgauthor = spacename
            spacemessage = Message(spacemsgauthor, spacemsg)
            spaces.append(Space(spaceid, spacename, spacemessage))
        self.driver.switch_to.default_content()
        return spaces

    def chat(self, message):
        """Send a message in a space"""
        if not self.is_in_space:
            raise StatusError("User is not in a space")
        typeui = self.driver.find_element(
            by=By.XPATH,
            value='//div[@jsname="yrriRe"]',
        )
        typeui.send_keys(message)
        typeui.send_keys(Keys.RETURN)

    def get_current_msg(self):
        """
        Gets the latest message in the space and outputs a Message object.
        This function is a bit buggy, but it works most of the time.
        If you want to have guaranteed accuracy, use the `ls` function.
        I may in fact deprecate this function in the future, but it's here for now.
        """
        if not self.is_in_space:
            raise StatusError("User is not in a space")
        page_html = self.driver.page_source
        soup = bs4.BeautifulSoup(page_html, "lxml")
        currentmsg = ""
        # todo: replies don't work
        try:
            for i in (
                soup.find_all(class_="rogmqd")[-1]
                .find("div", class_="DTp27d QIJiHb Zc1Emd")
                .contents
            ):
                if isinstance(i, bs4.element.NavigableString):
                    currentmsg += i
                if isinstance(i, bs4.element.Tag):
                    if i.name == "img":
                        if i.attrs.get("alt"):
                            currentmsg += i.attrs["alt"]
                    if i.name in ('b', 'i'):
                        if i.string:
                            currentmsg += i.string
        except (
            AttributeError
        ) as e:  # AttributeError: 'NoneType' object has no attribute 'contents'
            # this error happens when user sends ONLY image/file
            raise NoMessageFoundError("No messages found.") from e
        currentauthor = (
            soup.find_all(class_="nzVtF")[-1].find("span").find("span").string
        )
        if currentmsg and currentauthor:
            return Message(author=currentauthor, message=currentmsg)
        raise NoMessageFoundError("No messages found.")

    def list_messages(self):
        """Probably worse than `get_current_msg`"""
        if not self.is_in_space:
            raise StatusError("User is not in a space")
        page_html = self.driver.page_source
        soup = bs4.BeautifulSoup(page_html, "lxml")
        divs = soup.find_all("div", class_="F0wyae oGsu4")
        for div in divs:
            msg = ""
            nosend = False
            auth = div.find("span", class_="nzVtF").find("span").find("span").string
            # msg = (
            #     div.find("div", class_="rogmqd")
            #     .find("div", class_="DTp27d QIJiHb Zc1Emd")
            #     .string
            # )
            try:
                for i in (
                    div.find_all(class_="rogmqd")[-1]
                    .find("div", class_="DTp27d QIJiHb Zc1Emd")
                    .contents
                ):
                    if isinstance(i, bs4.element.NavigableString):
                        msg += i
                    if isinstance(i, bs4.element.Tag):
                        if i.name == "img":
                            if i.attrs.get("alt"):
                                msg += i.attrs["alt"]
                        if i.name in ('b', 'i'):
                            if i.string:
                                msg += i.string
            except (
                AttributeError
            ):  # AttributeError: 'NoneType' object has no attribute 'contents'
                # this error happens when user sends ONLY image/file
                nosend = True
            if not nosend:
                yield Message(author=auth, message=msg)

    def end(self):
        """Stops the driver and ends the session"""
        self.driver.quit()


class Message:
    def __init__(self, author, message):
        self.author = author
        self.message = message


class Space:
    def __init__(self, spaceid: str, name: str, message: Message):
        self.spaceid = spaceid
        self.name = name
        self.message = message


class StatusError(Exception):
    pass


class NoMessageFoundError(Exception):
    pass


class AuthorizationError(Exception):
    pass



#!/usr/bin/env python3
"""Google Chat for Character.AI"""

# SPDX-FileCopyrightText: 2025 NexusSfan <nexussfan@duck.com>
# SPDX-License-Identifier: GPL-3.0-or-later

import asyncio
import os
import pickle
import configparser
import time
import PyCharacterAI
import pyhangouts2

MODEL = input("Character? ")

WILL_REUSE = False

if os.path.exists("token.pkl"):
    WILL_REUSE = input("Reuse token? (y/N) ") == "y"

if WILL_REUSE:
    with open("token.pkl", "rb") as a:
        TOKEN = pickle.load(a)
else:
    TOKEN = input("Token? ")
    with open("token.pkl", "wb") as b:
        pickle.dump(TOKEN, b)

config = configparser.ConfigParser()

config.read("config.ini")

login_config = dict(dict(config).get("login", {}))

hangouts = pyhangouts2.PyHangouts2()

spacename = input('Space? (ex. "dm/XXXXXXXXXXX" or "space/XXXXXXXXXXX") ')

hangouts.login(
    login_config.get("method", "manual"),
    login_config.get("username"),
    login_config.get("password"),
)

hangouts.join(spacename)


async def characterai_chat(message: str, chat_id, character_id):
    client = await PyCharacterAI.get_client(token=TOKEN)
    me = await client.account.fetch_me()
    print(f"Authenticated as @{me.username}")
    airesponse = await client.chat.send_message(character_id, chat_id, message)
    aicontent = " ".join(airesponse.get_primary_candidate().text.split())
    with open("response.pkl", "wb") as c:
        pickle.dump(aicontent, c)
    await client.close_session()


async def begin_chat():
    client = await PyCharacterAI.get_client(token=TOKEN)
    me = await client.account.fetch_me()
    print(f"Authenticated as @{me.username}")
    chat, greeting_message = await client.chat.create_chat(MODEL)
    with open("chat.pkl", "wb") as d:
        pickle.dump({"chat": chat, "greeting_message": greeting_message}, d)
    await client.close_session()


asyncio.run(begin_chat())

with open("chat.pkl", "rb") as e:
    chatpkl = pickle.load(e)
cai_chat = chatpkl["chat"]
cai_greeting_message = chatpkl["greeting_message"]

old_message = ""
current_message = ""
author = ""

hangouts.chat(cai_greeting_message.get_primary_candidate().text)

time.sleep(3)

while True:
    try:
        currentmsg = hangouts.get_current_msg()

        current_message = currentmsg.message
        author = currentmsg.author

    except pyhangouts2.NoMessageFoundError:
        current_message = ""
        author = "You"

    # print("msgheader")
    # print(current_message)
    # print(old_message)
    # print(author)

    if current_message not in (old_message, "") and author != "You":
        sentmessage = f"{author}: {current_message}"
        # print(f"respond to {current_message}")
        asyncio.run(characterai_chat(sentmessage, cai_chat.chat_id, MODEL))
        with open("response.pkl", "rb") as f:
            response = pickle.load(f)
        hangouts.chat(response)
        ai_message = response
        old_message = current_message



"""Really bad code."""

# SPDX-FileCopyrightText: 2025 NexusSfan <nexussfan@duck.com>
# SPDX-License-Identifier: GPL-3.0-or-later

import curses
import sys

if __name__ == "__main__":
    raise RuntimeError("This file is not meant to be executed.")
options = []
realoptions = []


def update_options(upd):
    global options
    options = upd


def real_options(upd):
    global realoptions
    realoptions = upd


def select(stdscr):
    # Clear screen
    stdscr.clear()

    # Define menu options
    current_row = 0

    while True:
        # Display the menu
        for idx, option in enumerate(options):
            if idx == current_row:
                stdscr.addstr(
                    idx, 0, option, curses.A_REVERSE
                )  # Highlight the current option
            else:
                stdscr.addstr(idx, 0, option)

        stdscr.refresh()

        # Get user input
        key = stdscr.getch()

        # Navigate the menu
        if key == curses.KEY_UP and current_row > 0:
            current_row -= 1
        elif key == curses.KEY_DOWN and current_row < len(options) - 1:
            current_row += 1
        elif key == curses.KEY_ENTER or key in [10, 13]:  # Enter key
            break

    # Clear the screen before exiting
    stdscr.clear()
    stdscr.refresh()

    if realoptions[current_row] == "Exit":
        sys.exit()
    else:
        return realoptions[current_row]



#!/usr/bin/env python3
"""Generate config.ini"""

# SPDX-FileCopyrightText: 2025 NexusSfan <nexussfan@duck.com>
# SPDX-License-Identifier: GPL-3.0-or-later

import configparser
import sys

if __name__ != "__main__":
    raise RuntimeError("This file is not meant to be imported.")

if sys.argv[1] in ("-h", "--help"):
    print("Usage: python3 -m pyhangouts2.genconf [path (default: config.ini)]")
    sys.exit(0)

try:
    path = sys.argv[1]
except IndexError:
    path = "config.ini"

config = configparser.ConfigParser()

login_method = input("Login method? (auto/manual) ")
if login_method == "auto":
    config["login"] = {
        "method": "auto",
        "username": input("Username? "),
        "password": input("Password? "),
    }
elif login_method == "manual":
    config["login"] = {
        "method": "manual",
        "username": "",
        "password": "",
    }
else:
    raise SyntaxError("No valid login method provided!")

with open(path, "w", encoding="utf-8") as f:
    config.write(f)



#!/usr/bin/env python3
"""G-ChaTTY: A CLI for Google Chat."""

# SPDX-FileCopyrightText: 2025 NexusSfan <nexussfan@duck.com>
# SPDX-License-Identifier: GPL-3.0-or-later

import curses
import argparse
import os
from configparser import ConfigParser
import threading
import time
import pyhangouts2.selectoptions as so
import pyhangouts2

stop_threads = False

if __name__ != "__main__":
    raise RuntimeError("This file is not meant to be imported.")


def parse_args():
    parser = argparse.ArgumentParser(description="G-ChaTTY")
    parser.add_argument("--headless", action="store_true", help="Run in headless mode?")
    parser.add_argument(
        "-s", "--space", type=str, help="Space to automatically join.", default=None
    )
    parser.add_argument(
        "-c", "--config", type=str, help="Config file to use.", default="config.ini"
    )
    parser.add_argument(
        "-v",
        "--version",
        action="version",
        version=f"G-ChaTTY {pyhangouts2.__version__}",
    )
    args = parser.parse_args()
    return args


arguments = parse_args()

hangouts = pyhangouts2.PyHangouts2(headless=arguments.headless)

config = ConfigParser()

login_config = {}

# check if config exists
if os.path.exists(arguments.config):
    config.read(arguments.config)

    login_config = dict(dict(config).get("login", {}))

hangouts.login(
    login_config.get("method", "manual"),
    login_config.get("username"),
    login_config.get("password"),
)


def select_space():
    spaces = hangouts.ls()

    spacesrender = []
    spacesselect = []

    for space in spaces:
        spacesrender.append(
            f"{space.name} ({space.message.author}: {space.message.message})"
        )
        spacesselect.append(space.spaceid)

    spacesrender.append("Exit")
    spacesselect.append("Exit")

    so.update_options(spacesrender)
    so.real_options(spacesselect)

    to_join = curses.wrapper(so.select)

    hangouts.join(to_join)

    # clear screen
    print("\033[H\033[J", end="")


class stdout_thread(threading.Thread):
    def __init__(self):
        threading.Thread.__init__(self)

    def run(self):
        print("Welcome to G-ChaTTY. Type /? for help.")
        old_message = ""
        current_message = ""
        author = ""

        for i in hangouts.list_messages():
            print(f"{i.author}: {i.message}")
            old_message = i.message

        while True:
            if stop_threads:
                break
            try:
                currentmsg = hangouts.get_current_msg()

                current_message = currentmsg.message
                author = currentmsg.author

            except pyhangouts2.NoMessageFoundError:
                current_message = ""
                author = "You"

            if current_message not in (old_message, ""):
                sentmessage = f"{author}: {current_message}"
                print(sentmessage)
                old_message = current_message


if __name__ == "__main__":
    if arguments.space:
        hangouts.join(arguments.space)
    else:
        select_space()

    out_thread = stdout_thread()

    out_thread.start()

    while True:
        to_chat = input()
        if to_chat == "/back":
            stop_threads = True
            out_thread.join()
            hangouts.leave()
            time.sleep(1)
            select_space()
            # restart
            stop_threads = False
            out_thread = stdout_thread()
            out_thread.start()
            to_chat = ""
        if to_chat == "/exit":
            stop_threads = True
            out_thread.join()
            hangouts.end()
            break
        if to_chat in ('/?', '/help'):
            print("Commands:")
            print("/exit - Exit")
            print("/back - Back to space selection")
            print("/? - Help")
            to_chat = ""
        hangouts.chat(to_chat)



#!/usr/bin/env python3
"""Google Chat chatbot"""

# SPDX-FileCopyrightText: 2025 NexusSfan <nexussfan@duck.com>
# SPDX-License-Identifier: GPL-3.0-or-later

import configparser
import time
import ollama
import pyhangouts2

config = configparser.ConfigParser()

config.read("config.ini")

login_config = dict(dict(config).get("login", {}))

MODEL = input("Model? (default: llama3.1) ")

if MODEL == "":
    MODEL = "llama3.1"

hangouts = pyhangouts2.PyHangouts2()

spacename = input('Space? (ex. "dm/XXXXXXXXXXX" or "space/XXXXXXXXXXX") ')

hangouts.login(
    login_config.get("method", "manual"),
    login_config.get("username"),
    login_config.get("password"),
)

hangouts.join(spacename)


def ollama_chat(message: str, history: list):
    history.append({"role": "user", "content": message})
    airesponse = ollama.chat(model=MODEL, messages=history)
    aicontent = airesponse.message.content
    history.append({"role": "assistant", "content": aicontent})
    return aicontent, history


old_message = ""
current_message = ""
author = ""
aihistory = []

hangouts.chat("Hello World!")

time.sleep(3)

while True:
    try:
        currentmsg = hangouts.get_current_msg()

        current_message = currentmsg.message
        author = currentmsg.author
    except pyhangouts2.NoMessageFoundError:
        print("something broke")
        current_message = ""
        author = "You"

    # print("msgheader")
    # print(current_message)
    # print(old_message)
    # print(author)

    if current_message not in (old_message, "") and author != "You":
        sentmessage = f"{author}: {current_message}"
        print(f"respond to {current_message}")
        response, aihistory = ollama_chat(sentmessage, aihistory)
        hangouts.chat(response)
        ai_message = response
        old_message = current_message
